--	Code: Ragetti Art: Jerek
--	File: rMark.lua

rMark = {}

function rMark:New(...)
	local result = ZO_Object.New( self )
	
	result:Initialize()

	return result
end
  
function rMark:Initialize(...)
    self.addonName = "rMark"
	self.version = "1.3"
	self.em = GetEventManager()
	self.lam = LibStub('LibAddonMenu-1.0')
	self.leaderSymbolTypeControl=nil
	self.partySymbolTypeControl={}
	self.partyNameControl={}
	self.groupCharacterNames={}
	self.groupCharacterNameToUnitTag={}

	self:IntializeTextures()
	self:IntializeDefaults()
		
	self.em:RegisterForEvent(self.addonName.."_OnAddOnLoaded", EVENT_ADD_ON_LOADED, function(...) self:OnAddOnLoaded(...) end)
end

function rMark:IntializeTextures()
	self.textures = {}
	self.textures['General Star'] = {
		['Gold'] = "rMark/images/generalStarGold.dds",
		['Bronze'] = "rMark/images/generalStarBronze.dds",
		['Silver'] = "rMark/images/generalStarSilver.dds",
	}
	self.textures['Shield'] = {
		['Colored'] = "rMark/images/shieldColored.dds",
		['ESO'] = "rMark/images/shieldESO.dds",
	}	
	self.textures['Medic Cross'] = {
		['Blue'] = "rMark/images/medicCrossBlue.dds",
		['Green'] = "rMark/images/medicCrossGreen.dds",
		['Red'] = "rMark/images/medicCrossRed.dds",
	}
	self.textures['Chevron'] = {
		['White'] = "rMark/images/chevronWhite.dds",
		['Green'] = "rMark/images/chevronGreen.dds",
		['Yellow'] = "rMark/images/chevronYellow.dds",
	}
	self.textures['Medic Wings'] = {
		['Basic'] = "rMark/images/medicWings.dds",
	}	
	self.textures['Star Burst'] = {
		['Yellow'] = "rMark/images/starBurstYellow.dds",
		['Orange'] = "rMark/images/starBurstOrange.dds",
		['Red'] = "rMark/images/starBurstRed.dds",
	}	
	self.textures['Twin Axes'] = {
		['Colored'] = "rMark/images/twinAxeColored.dds",
		['ESO'] = "rMark/images/twinAxeESO.dds",
	}
	self.textures['Bullseye'] = {
		['Blue'] = "rMark/images/bullseyeBlue.dds",
		['Green'] = "rMark/images/bullseyeGreen.dds",
		['Red'] = "rMark/images/bullseyeRed.dds",
	}
	
	self.textureNames = self:GetKeyList(self.textures)
end

function rMark:IntializeDefaults()
	self.defaults = {
		markLeader = true,
		leaderSymbol = 'General Star',
		leaderSymbolType = 'Gold',
		compassPinMaxDistance = 0.10,
		maxMarkSize = 64,
		minMarkSize = 8,
		varyMarkSizeWithDistance = true,
		partyCount = 4,
		updateRate=2000,
		party= {},
	}
	for i = 1,self.defaults.partyCount do -- 
	    self.defaults.party[i] = {}
		self.defaults.party[i].enable = false
		self.defaults.party[i].symbol = 'Star Burst'
		self.defaults.party[i].symbolType = 'Yellow'
		self.defaults.party[i].unitName = nil
	end
end

function rMark:OnAddOnLoaded( event, addon )
	if addon ~= self.addonName then 
		return
	end
	self.em:UnregisterForEvent(self.addonName.."_OnAddOnLoaded", EVENT_ADD_ON_LOADED )
	
	self.settings = ZO_SavedVars:New( 'rMark_SavedVariables', self.version, nil, self.defaults )
	
	self.leaderSymbolTypes = self:GetKeyList(self.textures[self.settings.leaderSymbol])
	
	self.partySymbolTypes={}
	for i = 1,self.settings.partyCount do 
	    self.partySymbolTypes[i]= self:GetKeyList(self.textures[self.settings.party[i].symbol])
	end
	
	self:RefreshGroupNamesCache()
	self:CreateSettingsMenu()
	self:RegisterEvents()
	self:AddCustomPins()
	self:RefreshAllCompassPins()
	
	self.em:RegisterForUpdate(self.addonName.."_OnRefreshUpdate",self.settings.updateRate,function(...) self:OnRefreshUpdate(...) end)
end
 
function rMark:CreateSettingsMenu(...)
	self.controlPanelId = self.lam:CreateControlPanel('rMark_Settings', 'rMark')
	self.lam:AddHeader(self.controlPanelId, 'rMark_Settings_Header', '|cFFFF22rMark|r Party Marker')
	
	self.lam:AddSlider( self.controlPanelId, 'rMark_Settings_MaxCompassDistance', 'Max Distance', 'The maximum distance that the marks are seen.', 
		5, 100, 1,
        function() 
			local distance = self.settings.compassPinMaxDistance * 100
			return distance
		end, 
		function( newDistance ) 
			local distance = newDistance / 100
			self.settings.compassPinMaxDistance = distance
			self:AddCustomPins()
			self:RefreshAllCompassPins()
		end )
		
	self.lam:AddSlider( self.controlPanelId, 'rMark_Settings_MaxMarkSize', 'Max Mark Size', 'The maximum size that the mark will grow.', 
		8, 64, 2,
        function() 
			return self.settings.maxMarkSize
		end, 
		function( size ) 
			self.settings.maxMarkSize = distance 
		end )
		
	self.lam:AddCheckbox(self.controlPanelId, 'rMark_Settings_VaryByDistance', 'Vary By Distance Enable', 'If enabled the mark will size based on distance from player.',
		function() return self.settings.varyMarkSizeWithDistance end,
		function() 
			self.settings.varyMarkSizeWithDistance = not self.settings.varyMarkSizeWithDistance
		end,
		false, '')
		
	self.lam:AddSlider( self.controlPanelId, 'rMark_Settings_UpdateRate', 'Update Rate(ms)', 'The rate in which mark locations are updated.', 
		200, 3000, 100,
        function() 
			return self.settings.updateRate
		end, 
		function( newRate )
			self.em:UnregisterForUpdate(self.addonName.."_OnRefreshUpdate")	
			self.settings.updateRate=newRate
			self.em:RegisterForUpdate(self.addonName.."_OnRefreshUpdate",self.settings.updateRate,function(...) self:OnRefreshUpdate(...) end)	
		end )	
--------------------------------------------------------------------------------------------------------	
	self.lam:AddHeader(self.controlPanelId, 'rMark_Leader_Settings_Header', 'Leader Settings')
	self.lam:AddCheckbox(self.controlPanelId, 'rMark_Settings_Leader_Enable', 'Enable', 'Enable.',
		function() return self.settings.markLeader end,
		function() 
			self.settings.markLeader = not self.settings.markLeader
			self:RefreshAllCompassPins()
		end,
		false, '')	

	self.lam:AddDropdown(self.controlPanelId, "rMark_Settings_Leader_Symbol", "Select Marking Symbol", "Select the symbol to use for marking the leader",
				self.textureNames,
                function() return self.settings.leaderSymbol end,
				function(newSymbol)				
					-- Validate symbol
					self.settings.leaderSymbol,self.settings.leaderSymbolType=self:GetSymbolAndSymbolType(newSymbol,self.settings.leaderSymbolType)
					
					-- Refresh symbol types
					self.leaderSymbolTypes=self:GetKeyList(self.textures[self.settings.leaderSymbol])
					self.leaderSymbolTypeControl.valid=self.leaderSymbolTypes
					
					-- Update SymbolTypeDropDown
					ZO_OptionsWindow_InitializeControl(self.leaderSymbolTypeControl)
					local dropmenu = ZO_ComboBox_ObjectFromContainer(GetControl(self.leaderSymbolTypeControl, "Dropdown"))
					dropmenu:SetSelectedItem(self.settings.leaderSymbolType)	
					
					-- Update pins
					self:AddCustomPins()
					self:RefreshAllCompassPins()
				end,
				false,
				nil)

	self.leaderSymbolTypeControl = self.lam:AddDropdown(self.controlPanelId, "rMark_Settings_Leader_SymbolTypes", "Select Type Symbol", "Select the type of symbol to use for marking the leader",
				self.leaderSymbolTypes,
                function()
					-- Validate symbol
					self.settings.leaderSymbol,self.settings.leaderSymbolType=self:GetSymbolAndSymbolType(self.settings.leaderSymbol,self.settings.leaderSymbolType)
					
					return self.settings.leaderSymbolType 
				end,
				function(newSymbolType)	
					self.settings.leaderSymbolType = newSymbolType
					self:AddCustomPins()
					self:RefreshAllCompassPins()
				end,
				false,
				nil)				
--------------------------------------------------------------------------------------------------------				
	for i = 1,self.settings.partyCount do
		self.lam:AddHeader(self.controlPanelId, 'rMark_Settings_Header_Mark'..i, 'Mark '..i..' Settings')
		self.lam:AddCheckbox(self.controlPanelId, 'rMark_Settings_Enable_Mark'..i, 'Enable', 'Enable',
			function() return self.settings.party[i].enabled end,
			function()
				self:RefreshGroupNamesCache()
				self:RefreshPartyNameControls()
		
				self.settings.party[i].enabled = not self.settings.party[i].enabled
				self:RefreshAllCompassPins()
			end,
			false, '')	

		self.lam:AddDropdown(self.controlPanelId, 'rMark_Settings_SYMBOL_Player'..i, "Select Marking Symbol", "Select the symbol to use for marking",
					self.textureNames,
					function() return self.settings.party[i].symbol end,
					function(newSymbol)	
						-- Validate symbol
						self.settings.party[i].symbol,self.settings.party[i].symbolType=self:GetSymbolAndSymbolType(newSymbol,self.settings.party[i].symbolType)
						
						-- Refresh symbol types
						self.partySymbolTypes[i]=self:GetKeyList(self.textures[self.settings.party[i].symbol])
						self.partySymbolTypeControl[i].valid=self.partySymbolTypes[i]
						
						-- Update SymbolTypeDropDown
						ZO_OptionsWindow_InitializeControl(self.partySymbolTypeControl[i])
						local dropmenu = ZO_ComboBox_ObjectFromContainer(GetControl(self.partySymbolTypeControl[i], "Dropdown"))
						
						dropmenu:SetSelectedItem(self.settings.party[i].symbolType)
						
						-- Update pins
						self:AddCustomPins()
						self:RefreshAllCompassPins()
					end,
					false,
					nil)
					
		self.partySymbolTypeControl[i]=self.lam:AddDropdown(self.controlPanelId, 'rMark_Settings_SYMBOLType_Player'..i, "Select Type Symbol", "Select the type of symbol to use for marking",
				self.partySymbolTypes[i],
                function()		
					-- Validate symbol
					self.settings.party[i].symbol,self.settings.party[i].symbolType=self:GetSymbolAndSymbolType(self.settings.party[i].symbol,self.settings.party[i].symbolType)

					return self.settings.party[i].symbolType 
				end,
				function(newSymbolType)	
					self.settings.party[i].symbolType = newSymbolType
					
					-- Update pins
					self:AddCustomPins()
					self:RefreshAllCompassPins()
				end,
				false,
				nil)
					
		self.partyNameControl[i]=self.lam:AddDropdown(self.controlPanelId, 'rMark_Settings_Name_Player'..i, 'Select Party Member', 'Party members name to mark.',
				self.groupCharacterNames, 
				function() return self.settings.party[i].unitName end,
				function(newName) 
					self.settings.party[i].unitName = newName
					
					-- Update pins
					self:AddCustomPins()
					self:RefreshAllCompassPins()
				end,
				false,
				nil)			
		
	end
				
	self.lam:AddDescription(self.controlPanelId, 'rMark_Settings_Authors', '|c22FF22Ragetti|r, |c22FF22Jerek|r', '|cFFFF22Creators')
end
 
function rMark:RegisterEvents(...)
 	self.em:RegisterForEvent(self.addonName.."_OnGroupDisbanded",             	EVENT_GROUP_DISBANDED,             		function(...) self:OnGroup(...) end)
	self.em:RegisterForEvent(self.addonName.."_OnLeaderUpdate",             	EVENT_LEADER_UPDATE,               		function(...) self:OnGroup(...) end)
	self.em:RegisterForEvent(self.addonName.."_OnGroupMemberJoined",            EVENT_GROUP_MEMBER_JOINED,              function(...) self:OnGroup(...) end)
	self.em:RegisterForEvent(self.addonName.."_OnGroupMemberLeft",             	EVENT_GROUP_MEMBER_LEFT,               	function(...) self:OnGroup(...) end)
	self.em:RegisterForEvent(self.addonName.."_OnGroupMemberConnectedStatus",	EVENT_GROUP_MEMBER_CONNECTED_STATUS,    function(...) self:OnGroup(...) end)	
end

function rMark:OnGroup(...)
	self:RefreshGroupNamesCache()
	self:RefreshPartyNameControls()
	
	self:RefreshAllCompassPins()
end

function rMark:RefreshPartyNameControls()
	for i = 1,self.settings.partyCount do
		self.partyNameControl[i].valid={}
		self.partyNameControl[i].valid=self.groupCharacterNames
		ZO_OptionsWindow_InitializeControl(self.partyNameControl[i])
	end
end
  
function rMark:CompassSizeCallback(pin, angle, normalizedAngle, normalizedDistance)
	local size =self.settings.maxMarkSize
	if not pin then return end

	if self.settings.varyMarkSizeWithDistance then
		size = size - (normalizedDistance * 80)
		size = math.floor(size)
		if size < self.settings.minMarkSize then
			size = self.settings.minMarkSize
		end
	end
	
	pin:SetDimensions(size, size)
end
  
function rMark:OnLeaderCompassPinCallback(pinManager)
	if self.settings.markLeader then
		local leaderTag = GetGroupLeaderUnitTag()
		if leaderTag and GetUnitName(leaderTag) ~= GetUnitName("player") then
			local leaderX, leaderY = GetMapPlayerPosition(leaderTag)
			pinManager:CreatePin("rMark_leader", leaderTag, leaderX, leaderY)
		end
	end
end

function rMark:RefreshGroupNamesCache(...)
	self.groupCharacterNames={}
	self.groupCharacterNameToUnitTag={}
	self.groupCharacterNames, self.groupCharacterNameToUnitTag = self:GetGroupNames()
end

function rMark:GetGroupNames(...)
	local groupCharacterNames = {}
	local groupCharacterNameToUnitTag={}
	local groupUnitTagToCharacterName={}
	local size = GetGroupSize()
	for i=1, size do
		local unitTag = GetGroupUnitTagByIndex(i)
		local groupCharacterName = GetUnitName(unitTag)
		groupCharacterNames[i]=groupCharacterName
		groupCharacterNameToUnitTag[groupCharacterName]=unitTag
		groupUnitTagToCharacterName[unitTag]=groupCharacterName
	end
	return groupCharacterNames,groupCharacterNameToUnitTag,groupUnitTagToCharacterName
end

function rMark:GetKeyList(aTable)
	local names = {}
	local n=1
	for k,v in pairs(aTable) do
		names[n]=k
		n=n+1
	end
	table.sort(names)
	return names	
end

function rMark:GetSymbolAndSymbolType(symbol,symbolType)
	local textureGroup = self.textures[symbol]
	
	if textureGroup==nil then
		textureGroup = self.textures[self.defaults.leaderSymbol]
		symbol = self.defaults.leaderSymbol
	end

	local aTexture = textureGroup[symbolType]
	if not aTexture then
		local symbolTypeNames = self:GetKeyList(textureGroup)
		symbolType=symbolTypeNames[1]
	end

	return symbol,symbolType
end

function rMark:OnPartyPinCallback(pinManager,index)
	if self.settings.party[index].enabled and self.settings.party[index].unitName then
		local unitName = self.settings.party[index].unitName
		local unitTag = self.groupCharacterNameToUnitTag[unitName]
		if not unitTag then
			return 
		end
		
		-- leader takes precendence
		local leaderTag = GetGroupLeaderUnitTag()
		if leaderTag and GetUnitName(leaderTag) ~= GetUnitName(unitTag) then
			local x, y = GetMapPlayerPosition(unitTag)
			pinManager:CreatePin("rMark_party"..index, unitTag, x, y)			
		end
	end
end
  
function rMark:RefreshAllCompassPins()
	COMPASS_PINS:RefreshPins("rMark_leader")
	for i = 1,self.settings.partyCount do
		COMPASS_PINS:RefreshPins("rMark_party"..i)
	end
end
  
function rMark:AddCustomPins()
	self.settings.leaderSymbol,self.settings.leaderSymbolType=self:GetSymbolAndSymbolType(self.settings.leaderSymbol,self.settings.leaderSymbolType)
	local aTexture = self.textures[self.settings.leaderSymbol][self.settings.leaderSymbolType]
	
  	local compassPinLayout = {
		maxDistance = self.settings.compassPinMaxDistance,
		texture = aTexture,
		sizeCallback = function(...) self:CompassSizeCallback(...) end,
    }	
	COMPASS_PINS:AddCustomPin("rMark_leader", function(...) self:OnLeaderCompassPinCallback(...) end, compassPinLayout)
	
	for i = 1,self.settings.partyCount do
		if self.settings.party[i].enabled then
			self.settings.party[i].symbol,self.settings.party[i].symbolType=self:GetSymbolAndSymbolType(self.settings.party[i].symbol,self.settings.party[i].symbolType)
			aTexture = self.textures[self.settings.party[i].symbol][self.settings.party[i].symbolType]

			local partyCompassPinLayout= {
				maxDistance = self.settings.compassPinMaxDistance,
				texture = aTexture,
				sizeCallback = function(...) self:CompassSizeCallback(...) end,
			}
			COMPASS_PINS:AddCustomPin("rMark_party"..i, function(pinManager) self:OnPartyPinCallback(pinManager,i) end, partyCompassPinLayout)
		end
	end
	
	--self:Debug()
end

function rMark:OnRefreshUpdate(...)
	self:RefreshAllCompassPins()
end

function rMark:Debug()
	local partyCompassPinLayout= {
		maxDistance = self.settings.compassPinMaxDistance,
		texture = "rMark/images/chevronWhite.dds",
		sizeCallback = function(...) self:CompassSizeCallback(...) end,
	}
	COMPASS_PINS:AddCustomPin("rMark_debug", function(...) self:OnDebugCompassPinCallback(...) end, partyCompassPinLayout)
end

function rMark:OnDebugCompassPinCallback(pinManager)
	local unitTag = 'player'

	local x, y = GetMapPlayerPosition(unitTag)

	pinManager:CreatePin("rMark_debug", unitTag, x, y)
end


rMark:New()
 